import org.openqa.selenium.*;
import org.openqa.selenium.remote.RemoteWebDriver;

import java.util.*;

/**
 * @author V. Mihashcenko
 * @version 1.1
 *
 * Class for authorization at linkedin.com
 * Authorization is done thru cookie or email and password
 * After the authorization cookie are stored for the following authorizations 
 */

public final class AuthorizationInLinkedin
{
    public static final int NOT_COMPANY_DIRECTORY = -1;
    public static final int AUTHORIZATION_ERROR = -2;

    private static String email = "";
    private static String password = "";

    public static void setEmailAndPassword(String email, String password)
    {
        AuthorizationInLinkedin.email = email;
        AuthorizationInLinkedin.password = password;
    }

    /**
     * Authorization at Linkedin
     *      1. Set up cookie from array "cookie"
     *      2. Trying to get the page: https://www.linkedin.com/directory/companies
     *      3. If failed to authorized, it tries to authorize using 'static email' and 'static password'
     *          I. Again trying to get the target url
     *      4. Save cookie in the array "forSaveCookie"
     * @return >0, if successful authorization, and <0, if authorization failed
     */
    public static int authorizationLinkedin(RemoteWebDriver driver, ArrayList<Cookie> cookie, ArrayList<Cookie> forSaveCookie)
    {
        if(driver == null)
            throw new NullPointerException("driver = null");
        if(cookie == null)
            throw new NullPointerException("cookie = null");
        if(forSaveCookie == null)
            throw new NullPointerException("forSaveCookie = null");

        driver.navigate().to("https://www.linkedin.com");
        try
        {
            for (Cookie c : cookie)
                driver.manage().addCookie(c);
        }
        catch(Exception e) {System.out.println(e.getMessage());}

        driver.navigate().to("https://www.linkedin.com/directory/companies");

        /* search for an element, that is at the authorization page */
        if(driver.findElementsById("join-form-submit").size() == 0)
        {
            /* search for a match with the companies list page */
            List<WebElement> checkNextPage = driver.findElementsByXPath("//*[@id=\"seo-dir\"]/div/div[@class=\"page-title\"]/h1");
            if(checkNextPage.size() == 0 || !checkNextPage.get(0).getText().trim().equals("Company Directory"))
                return -1;

            /* проверка на авторизацию пользователя */
            if(driver.findElementsById("minimal-nav-account-btn").size() == 0)
            {
                if(!writeEmailAndPassword(driver))
                    return -2;
            }
        }
        else
            if(!writeEmailAndPassword(driver))
                return -2;

        forSaveCookie.addAll(driver.manage().getCookies());

        return 1;
    }

    /**
     * Authorization at Linkedin
     *      1. Set up cookie
     *      2. Trying to get the page: https://www.linkedin.com/directory/companies
     *      3. If failed to authorized, it tries to authorize using 'static email' and 'static password'
     *          I. Again trying to get the target url
     *      4. Save cookie in the array "cookies"
     * @return >0, if successful authorization, and <0, if authorization failed
     */
    public static int authorizationLinkedin(RemoteWebDriver driver, ArrayList<Cookie> cookies)
    {
        return authorizationLinkedin(driver, cookies, cookies);
    }

    /* ************************************************************************************ */

    private static boolean writeEmailAndPassword(RemoteWebDriver driver)
    {
        if(email == null || password == null || email.equals("") || password.equals(""))
        {
            System.err.println("XXXX The field 'password' or 'email' is empty XXXX");
            return false;
        }

        List<WebElement> elements;

        driver.navigate().to("https://www.linkedin.com/authwall");
        elements = driver.findElementsByXPath("//*/form/section/p/a");
        if(elements.size() > 0)
            elements.get(0).click();

        elements = driver.findElementsByName("session_key");
        if(elements.size() > 0)
            elements.get(0).sendKeys(email);
        else
            return false;

        elements = driver.findElementsByName("session_password");
        if(elements.size() > 0)
            elements.get(0).sendKeys(password);
        else
            return false;

        elements = driver.findElementsById("login-submit");
        if(elements.size() > 0)
            elements.get(0).click();
        else
            return false;

        driver.navigate().to("https://www.linkedin.com/directory/companies");
        List<WebElement> checkNextPage = driver.findElementsByXPath("//*[@id=\"seo-dir\"]/div/div[@class=\"page-title\"]/h1");
        if(checkNextPage.size() == 0
                || !checkNextPage.get(0).getText().trim().equals("Company Directory")
                || driver.findElementsById("minimal-nav-account-btn").size() == 0)
            return false;
        else
            return true;
    }

    /* ************************************************************************************ */

    private AuthorizationInLinkedin() {}
}
