package files;

import org.openqa.selenium.OutputType;
import org.openqa.selenium.TakesScreenshot;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.chrome.ChromeDriver;
import org.openqa.selenium.chrome.ChromeDriverService;
import org.openqa.selenium.chrome.ChromeOptions;

import javax.imageio.ImageIO;
import java.io.File;
import java.io.IOException;

/**
 * @author V. Mihashcenko
 * @version 1.0
 *
 * Method "pageScreen()" to make a screenshot of the screen 
 * Method  "navigate()". It makes possibile to pause, variable "pause".
 */

public final class ChromeDriverDecorator extends ChromeDriver
{
    private long pause = 0;
    public void setPause(long pause)
    {
        if(pause >= 0)
            this.pause = pause;
    }

    /* ************************************************************************************ */

    /**
     * @return a set up ChromeDriverDecorator
     */
    public static ChromeDriverDecorator createChromeDriverDecorator(String chromeDriverPath, String chromeBrowserPath, int widthWindow, int heightWindow, boolean headless)
    {
        System.setProperty("webdriver.chrome.driver", chromeDriverPath);
        ChromeOptions chromeOptions = new ChromeOptions();
        chromeOptions.setBinary(chromeBrowserPath);
        if(headless)
            chromeOptions.addArguments("--headless");
        chromeOptions.addArguments("--enable-javascript");
        chromeOptions.addArguments("lang=en");
        chromeOptions.addArguments("window-size=" + widthWindow + "x" + heightWindow);
        return new ChromeDriverDecorator(chromeOptions);
    }

    /* ************************************************************************************ */

    /**
     * @param "outputFile" should also have an image format at the end
     */
    public void pageScreen(String outputFile, String formatName) throws IOException
    {
        if(outputFile == null || outputFile.trim().equals(""))
            return;

        outputFile = outputFile.trim();
        File screen = ((TakesScreenshot) this).getScreenshotAs(OutputType.FILE);
        if(!ImageIO.write(ImageIO.read(screen), formatName, new File(outputFile)))
            throw new IOException("Screen did not save");
    }

    /* ************************************************************************************ */

    public WebDriver.Navigation navigate()
    {
        if(pause > 0)
        {
            try
            {
                synchronized (this)
                {
                    this.wait(pause);
                }
            }
            catch (InterruptedException e) {e.printStackTrace();}
        }
        return super.navigate();
    }

    /* ************************************************************************************ */

    public ChromeDriverDecorator()
    { super(); }

    public ChromeDriverDecorator(ChromeDriverService service)
    { super(service); }

    public ChromeDriverDecorator(ChromeDriverService service, ChromeOptions options)
    { super(service, options); }

    public ChromeDriverDecorator(ChromeOptions options)
    { super(options); }
}
