import files.ChromeDriverDecorator;
import files.LinkedinData;
import files.ScrapeLinkedinCompanyData;
import org.openqa.selenium.Cookie;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Properties;
import java.util.concurrent.TimeUnit;

/*
    1. Разделить большие методы
    2. Добавить вместо некоторых false - исключения
    3. Добавить проверки в методах на входящие данные
    4. Заменить возврат значение boolean на int в методе по парсингу страницы, чтобы точно понимать проблему
 */

/**
 * @author V. Mihashcenko
 * @version 1.0
 */

public class mainExample
{
    /* *******************************************************
     *  Comments signs:
     *    '#' - start, end
     *    '!' - attention
     *    'X' - critical failure (complete the process)
     *    '-' - minor failure (continue the process)
     *    '+' - successful completion
     ******************************************************* */

    public static void main(String[] args) throws IOException
    {
        //System.setOut(new PrintStream(new FileOutputStream("C:\\scrapeBusinessDirectoryJava\\txtFiles\\log.txt", true)));

        System.out.println("##### START #####");
        SimpleDateFormat showDate = new SimpleDateFormat("dd.MM.yyyy HH:mm");
        Date start = new Date();
        System.out.println(showDate.format(start));
        System.out.println();

        /* get email and password from file */
        try(FileInputStream readProperties = new FileInputStream("C:\\scrapeBusinessDirectoryJava\\txtFiles\\data.properties"))
        {
            Properties properties = new Properties();
            properties.load(readProperties);
            AuthorizationInLinkedin.setEmailAndPassword(properties.getProperty("email"), properties.getProperty("passwordLinkedin"));
        }
        catch(IOException e) {e.printStackTrace(); return;}

        ArrayList<Cookie> linkedinCookie = linkedinCookie(new File("C:\\scrapeBusinessDirectoryJava\\txtFiles\\cookie\\linkedin.txt"));
        HashMap<String, String> crunchBaseCookie = crunchBaseCookie(new File("C:\\scrapeBusinessDirectoryJava\\txtFiles\\cookie\\crunchbase.txt"));

        /* get "driver" and set it up */
        ChromeDriverDecorator driver = ChromeDriverDecorator.createChromeDriverDecorator("C:\\scrapeBusinessDirectoryJava\\driver\\chrome.exe",
                "C:\\Users\\Admin\\AppData\\Local\\Google\\Chrome\\Application\\chrome.exe",
                1366, 768, true);
        driver.manage().timeouts().implicitlyWait(10, TimeUnit.SECONDS);
        driver.setPause(500);

        try
        {
            if(AuthorizationInLinkedin.authorizationLinkedin(driver, linkedinCookie) < 0)
                System.out.println("XXXX Неудача авторизации XXXX");
            else
            {
                System.out.println("++++ Авторизация прошла успешно ++++");

                ScrapeLinkedinCompanyData scrapeLinkedin = new ScrapeLinkedinCompanyData(driver, crunchBaseCookie);

                /* Examples */
                //LinkedinData linkedinData;
                //linkedinData = scrapeLinkedin.scrapeData("https://www.linkedin.com/company/microsoft/about/");
                //linkedinData = scrapeLinkedin.scrapeData("https://www.linkedin.com/company/forbes-magazine/about/");
                //linkedinData = scrapeLinkedin.scrapeData("https://www.linkedin.com/company/netflix/about/");
                //ArrayList<String> links = scrapeLinkedin.scrapeCompaniesURL("https://www.linkedin.com/directory/companies");
                //System.out.println(linkedinData);
            }
        }
        catch (Exception e) {e.printStackTrace();}
        finally
        {
            try { driver.pageScreen("end.png", "png");}
            catch (IOException e) {e.getMessage();}
            driver.quit();

            Date end = new Date();
            long time = end.getTime() - start.getTime();
            float minutes = ((float) time) / 1000 / 60;

            System.out.println();
            System.out.println(showDate.format(end) + " (" + String.format("%.2f", minutes) + " minutes)");
            System.out.println("##### END #####");
            System.out.println();
            System.out.println();
            System.out.println();
        }
    }

    private static ArrayList<Cookie> linkedinCookie(File fileWithCookie) throws IOException
    {
        ArrayList<Cookie> result = new ArrayList<>();
        try(BufferedReader buffR = new BufferedReader(new FileReader(fileWithCookie)))
        {
            String name;
            while((name = buffR.readLine()) != null)
            {
                if(name.equals(""))
                    continue;

                String value = buffR.readLine();
                String domain = buffR.readLine();
                String path = buffR.readLine();
                String date = buffR.readLine();
                String isSecureStr = buffR.readLine();
                String isHttpOnlyStr = buffR.readLine();

                if(value == null || domain == null || path == null || date == null || isSecureStr == null || isHttpOnlyStr == null)
                    break;

                Date expire;
                if(date.equals("null"))
                    expire = null;
                else
                    expire = new Date(Long.parseLong(date));
                boolean isSecure = Boolean.parseBoolean(isSecureStr);
                boolean isHttpOnly = Boolean.parseBoolean(isHttpOnlyStr);

                Cookie c = new Cookie(name, value, domain, path, expire, isSecure, isHttpOnly);
                result.add(c);
            }
        }
        return result;
    }

    private static HashMap<String, String> crunchBaseCookie(File fileWithCookie) throws IOException
    {
        HashMap<String, String> result = new HashMap<>();
        try(BufferedReader buffR = new BufferedReader(new FileReader(fileWithCookie)))
        {
            String name;
            String value;
            while((name = buffR.readLine()) != null && (value = buffR.readLine()) != null)
            {
                if(name.equals("") || value.equals(""))
                    continue;

                String[] aName = name.split(":");
                String[] aValue = value.split(":");
                if(aName.length == 2 && aValue.length == 2)
                    result.put(aName[1], aValue[1]);
            }
        }
        return result;
    }
}
